<?
	/*	Yrch! - Hierarchised Website Directories
    Copyright (C) 2002-2003 Guillaume Boddaert <narya@yrch.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.*/
	
	class config
	{		
		/**
		* Tableau de stockage des informations de configuration
		*
		* @var		array
		* @access	private
		*/
		
		var $tab	= array();
		
		var $locked	= array();
		
		/**
		* Constructeur
		*
		* @param	$file	string  chemin vers un fichier de configuration (optionel)
		* @return	boolean         l'action s'est elle passe correctement
		* @since 	0.6
		*/
		function config($file = false)
		{
			if($file)
			{
				$this->load_file($file);
			}
		}
		
		/**
		* Dfinition de configuration sur base de fichier
		*
		* le fichier doit tre une source PHP et comporter un tableau associatif appel $c
		* les donnes sont ensuite charges dans cette classe.
		*
		* @param	$file	string	chemin vers le fichier de configuration
		* @return	boolean		l'action s'est elle passe correctement
		* @since 	0.6
		* @access	public
		*/
		function load_file($file, $prefix = false)
		{
			if(!file_exists($file))
			{
				return false;
			}
			
			if(!@include($file))
			{
				return false;
			}
			
			if(isset($c) && is_array($c))
			{
				while(list($key,$val) = each($c))
				{
					$this->define($key, $val, $prefix);
				}
			}			
			return true;
		}
		
		/**
		* Dfinition d'un valeur
		*
		* Si la cl dfinie commence par un underscore '_' alors il s'agit d'une variable
		* protege qui ne peut pas tre ecrase par le chargement d'un autre fichier de configuration
		*
		* @param	mixed	$key	la cl  introduire
		* @param	mixed	$value	la valeur correspondante  cette cl.
		* @return	boolean		l'action s'est elle passe correctement
		* @since 	0.6
		* @access	public
		*/
		function define($key, $value = "", $prefix = false)
		{		
			$lockit	= false;
			$key	= strtoupper($key);
			
			if(eregi("^_",$key) && !$this->is_locked($key))
			{				
				$key	= preg_replace("|^_|","",$key);
				$lockit	= true;
			}
			
			$chose	= $this->_set($prefix.$key,$value);
			
			if($lockit)
				$this->_lock($key);
				
			return $chose;
		}
		
		/**
		* Dfinition d'une valeur en interne
		*
		* @access	private
		* @param	string	$key	cl  dfinir
		* @param	mixed	$value	valeur de la clef
		* @since 	0.6
		*/
		function _set($key, $value)
		{			
			if($this->is_locked($key))	return false;
			
			$this->tab[$key] = $value;
			return true;
		}
		
		/**
		* Vrouiller une cle
		*
		* @param	string	la clef  vrouiller
		* @return	bool
		* @since 	0.6
		* @access	private
		*/
		function _lock($key)
		{
			$this->locked[$key]	= true;
			return true;
		}
		
		/**
		* Vrifier l'tat d'une clef
		*
		* Retourne vrai dans la cas o la variable indique est vrouille, faux
		* en cas d'absence ou de non-vrouillage.
		*
		* @param	string	la clef  vrouiller
		* @return	bool	
		* @access	public
		* @since 	0.6	
		*/
		function is_locked($key)
		{
			return in_array($key,array_keys($this->locked));	
		}
		
		/**
		* Dvrouiller une cle
		*
		* @param	string	la clef  vrouiller
		* @return	bool	succs ou erreur ?
		* @access	private
		* @since 	0.6
		*/
		function _unlock($key)
		{
			if(isset($this->locked[$key]))
			{
				$this->locked[$key]	= false;
				return true;
			}
			else	return false;
		}
		
		/**
		* Classer le tableau des clefs par ordre alphabtique
		*
		* @access	private
		*/
		function _arrange()
		{
			ksort($this->tab);
			reset($this->tab);
		}
		
		/**
		* Recupration d'une valeur
		*
		* Si la valeur correspondante  la cl prcde d'un underscore "_" est dfinie 
		* alors on la retourne ceci permet de recuprer une variable protege.
		*
		* @param	mixed	$key	la cl  rcuprer
		* @return	mixed		la valeur ou faux si l'opration a chou
		* @access	public
		* @since 	0.6
		*/ 
		function get($key)
		{	
			$key	= strtoupper($key);
						
			
			if(isset($this->tab[$key]))
			{
				return $this->tab[$key];
			}
			else
			{
				trigger_error(sprintf(_("Accs  une cl non dfinie (%s)"),$key), E_USER_NOTICE);
				return false;
			}
		}
		
		/**
		* Verification de l'xistance d'une cle
		*
		* Afin d'viter l'avertissement li  l'accs  une clef absente, vous avez la possibilit de tester son existence avant d'y accder grace  cette mthode. 
		* Vous pouvez passez un tableau ou une simple chaine. Si la moindre valeur est absente la fonction retourne faux.
		* Vous avez galement la possibilit de forcer un avertissement en cas d'absence grce au deuxime paramtre.
		*
		* @param	mixed	$keys	une clef ou un tableau de clefs  tester
		* @param	boolean	$warn	avertir automatiquement en cas d'absence de la clef/ d'une des clefs
		* @return	boolean			toutes les cls sont prsentes
		* @access	public
		* @since 	0.6
		*/
		function test($keys,$warn = false)
		{
			$return	= true;
						
			if(is_array($keys))
			{
				foreach($keys as $key)
				{
					$key	= strtoupper($key);
					if(!isset($this->tab[$key]))
					{
						$return = false;
						if($warn)	trigger_error(sprintf(_("La propriet '%s' est ncessaire, pourtant elle n'a pas t renseigne"),$key),E_USER_WARNING);
					}
				}
			}
			else
			{
				$key	= strtoupper($keys);
				if(!isset($this->tab[$key]))
				{
					$return = false;
					if($warn)	trigger_error(sprintf(_("La propriet '%s' est ncessaire, pourtant elle n'a pas t renseigne"),$key),E_USER_WARNING);
				}
			}
				
			return $return;
		}
		
		/**
		* Chercher une/des clef(s) via expression regulire PERL
		* 
		* On effecture une recherche par  expression rgulire de type PERL   dans
		* les clefs du tableau de configuration. Cette mthode retourne les  clefs
		* trouves.
		*
		* @param	string	$reg_exp	l'expression regulire PERL
		* @return	mixed	les valeurs trouves
		* @see		preg_grep()
		* @access	public
		* @since 	0.6
		*/
		function find($regexp)
		{			
			$matchs	= preg_grep($regexp,array_keys($this->tab));
			
			if(!$matchs)	return array();
			return $matchs;
		}
		
		/**
		* Liste des clefs dfinies
		*
		* Permet de lister l'ensemble des clefs actuellement dfinies.
		*/
		function keys()
		{
			$this->_arrange();
			return array_keys($this->tab);
		}
		
		/**
		* Affiche l'tat de la configuration
		*
		* Un tableau HTML est renvoy par cette mthode et permet de connaitre l'tat de la configuration
		* @return	string	la chaine du tableau HTML
		*/
		function dump()
		{
			$html	=  "<hr><FONT size=2 face='Sans-Serif'>Dump de la configuration</font>\n<P>\n\n";
			$html	.= "<TABLE BORDER=1 CELLPADDING=1 CELLSPACING=0>\n";
			$html	.= "<TR BGCOLOR='#000000'><TD><FONT COLOR='white' size=1 face='Sans-Serif'><B>CLEF</B></FONT></TD><TD><FONT COLOR='white' size=1 face='Sans-Serif'><B>VALEUR</B></FONT></TD></TR>";
			
			$buf_sav	= ob_get_contents();
				ob_end_flush();
			
			foreach($this->keys() as $cle)
			{
				ob_start();
				var_dump($this->get($cle));
				$ret_str = ob_get_contents();
				ob_end_clean();
				
				$html	.= "<tr BGCOLOR='#FFFFFF'><TD valign=\"top\"><FONT COLOR='black' size=1 face='Sans-Serif'>".$cle."</FONT></TD>\n";
				$html	.= "<TD><PRE><FONT COLOR='black' size=1 face='Sans-Serif'>".$ret_str."</font></PRE></TD></TR>\n";
			}
			$html	.= "<tr BGCOLOR='#000000'><TD colspan=2><FONT COLOR='white' size=1 face='Sans-Serif'>".count($this->keys())." cl(s)</font></tD></tr>\n</table>";
			$html	.= "<font size=1 face='Sans-Serif'>pour dsactiver cette fonctionnalit passez la variable de configuration \$c[\"FLAG_CONFIGDUMP\"]  faux (0)</font>\n";
			
			echo $buf_sav;
			
			return $html;
		}
	}
	
?>