<?php
	/*	Yrch! - Hierarchised Website Directories
    Copyright (C) 2002-2003 Guillaume Boddaert <narya@yrch.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.*/
        
        /**
		* Retourne la heure courante, mesure en secondes et millisecondes depuis le dbut de l'poque UNIX, (1er janvier 1970 00:00:00 GMT).
		*
		* Retourne l'quivalent de time() en beaucoup plus prcis
		*
		* @return	double	Temps en seconde, et millisecondes depuis le debut de l're UNIX
		* @see		microtime()
		*/
		
		function write_cache($file, $value)
		{
			global $yrch;
			
			$file	= $yrch->config->get("PATH_CACHE")."/".$yrch->config->get("YRCH_THEME").".".$yrch->config->get("LANG").".".$file;
			
			if(!$yrch->config->get("YRCH_CACHE"))	return false;
			
			if(strlen($value) <= 0) return false;
			
			$fp	= @fopen($file,"w+");
			if($fp)
			{
				@fwrite($fp, $value);
				@fclose($fp);
				return true;
			}
			return false;
		}
		
		function use_cache($file, $alias, $append = false, $time = false)
		{
			global $yrch, $yrch_layout;
			
			$file	= $yrch->config->get("PATH_CACHE")."/".$yrch->config->get("YRCH_THEME").".".$yrch->config->get("LANG").".".$file;
			
			if(!$time)			$time	= $yrch->config->get("CACHE_TTL");
			if(!$yrch->config->get("YRCH_CACHE"))	return false;
			
			if(!file_exists($file) || (time() - filemtime($file)) > $time)
			{
				return false;	
			}
			else
			{
				$yrch->event("rechargement du cache \"$file\"");
				$yrch_layout->set_file($file,$file, true);
				$yrch_layout->parse($alias, $file, $append);
				return true;
			}
		}
		
		function micronow($time	= false)
		{
			if($time == false)
				 $time = microtime();
						
			list($usec, $sec) = explode(" ",$time); 
    		$res	= ((float)$usec + (float)$sec);
    		return $res;//str_replace(".",",",$res);
		}
		
		function median($array)
		{
			if(!is_array($array))	return false;
			
			$array	= array_values($array);
			$count	= count($array);
			$median	= floor($count/2)-1;
			
			if(isset($array[$median]))
			{
				return $array[$median];
			}
			return false;
		}
        
        function &site_stack($id)
		{		
			global $yrch;
			
			if(!is_numeric($id))	return false;
			
			if(!isset($yrch->obj_index["site"][$id]))
			{
				$yrch->obj_index["site"][$id]	= new y_site($id);
				$yrch->fill($yrch->obj_index["site"][$id]);
			}
			return $yrch->obj_index["site"][$id];
		}
		
		function &user_stack($id)
		{		
			global $yrch;
			
			if(!is_numeric($id))	return false;
			
			if(!isset($yrch->obj_index["user"][$id]))
			{
				$yrch->obj_index["user"][$id]	= new y_user($id);
				$yrch->fill($yrch->obj_index["user"][$id]);
			}
			return $yrch->obj_index["user"][$id];
		}
		
		function &cat_stack($id)
		{		
			global $yrch;
			
			if(!is_numeric($id))	return false;
			
			if(!isset($yrch->obj_index["cat"][$id]))
			{
				$yrch->obj_index["cat"][$id]	= new y_cat($id);
				$yrch->fill($yrch->obj_index["cat"][$id]);
			}
			return $yrch->obj_index["cat"][$id];
		}
		
		function lang_stack($id)
		{
		    global $yrch;
		    
		    if(!is_numeric($id))	return false;
			
			if(!isset($yrch->obj_index["lang"][$id]))
			{
				$yrch->db->squery("LANG_READ",array($id));
				if($yrch->db->next_record())
				{
			        $yrch->obj_index["lang"][$id]   = $yrch->db->record;
			    }
			}
			return $yrch->obj_index["lang"][$id];
		}
		
		function shorten($string,$length,$suffix = "...")
		{
			if(strlen($string)	>= $length)
			{
				$string	= substr($string,0,($length-strlen($suffix))).$suffix;
			}	
			return $string;
		}
		
		function forcelen($str, $length, $mode = "right", $char = " ")
		{
			if(strlen($str) < $length)
			{
				while(strlen($str) < $length)
				{
					if($mode == "left")	$str	= $char.$str;
					else				$str	= $str.$char;
				}
				return $str;
			}
			else	return shorten($str,$length);
		}
		
		/**
		* Format file size in a human-readable way
		* such as "xx Megabytes" or "xx Mo"
		*
		* @author Andrea Paleni <andreaSPAMLESS_AT_SPAMLESScriticalbit.com>
		* @version 1.0
		* @param int bytes is the size
		* @param bool base10 enable base 10 representation, otherwise
		* default base 2 is used
		* @param int round number of fractional digits
		* @param array labels strings associated to each 2^10 or
		* 10^3(base10==true) multiple of base units
		*/
		function hbytes ($bytes, $base10=false, $round=0,$labels=array('bytes', 'Kb', 'Mb', 'Gb', 'Tb')) 
		{
			if (($bytes <= 0) || (! is_array($labels)) || (count($labels) <= 0))	return null;
			
			$step = $base10 ? 3 : 10 ;
			$base = $base10 ? 10 : 2;
			
			$log = (int)(log10($bytes)/log10($base));
			
			krsort($labels);
			
			foreach ($labels as $p=>$lab) {
				$pow = $p * $step;
				if ($log < $pow) continue;
				$text = round($bytes/pow($base,$pow),$round) . $lab;
				break;
			}
			
			return $text;
		}
		
		function html_decode($str)
		{
		    $trans = array_flip(get_html_translation_table(HTML_ENTITIES));
            return strtr($str, $trans);
		}
		
		function yrch_message($layout_element, $title, $message, $links = false)
		{
			global $yrch, $yrch_layout;
			
			if($links == false || !is_array($links))
			{
				$links	= array($yrch->config->get("YRCH_ROOT") => _("Retour  l'index"));
			}
			
			if(!$yrch->config->get("YRCH_TEMPLATEMESSAGE"))
		    {
		    	$yrch_layout->set_file("yrch_message.tpls","structure/yrch_message.tpls");
			    $yrch_layout->set_block("yrch_message.tpls","YRCH_MLINKS","yrch_mlinks.blk");
			    $yrch->config->define("YRCH_TEMPLATEMESSAGE",true);
			}

		    $yrch_layout->set_var("yrch_mlinks.blk","");
		    foreach($links as $location => $label)
		    {	    	
		    	$yrch_layout->set_var(array(
		    			"yrch_mlinks_location"	=> $location,
		    			"yrch_mlinks_label"		=> $label,
		    	));
		    	
		    	$yrch_layout->parse("yrch_mlinks.blk","YRCH_MLINKS",1);
		    }

			$yrch_layout->set_var(array(
		    		"yrch_mtitle"	=> $title,
		    		"yrch_mmessage"	=> $message
		    ));
	    
	    	$yrch_layout->parse($layout_element, "yrch_message.tpls");
	    	
	    	return true;
		}
		
		function yrch_edito($ref, $lang = false)
		{
            global $yrch;
            
            $exts   = array();
            $exts["images"] = array("jpg","gif","jpeg","png","tiff");
            $stream = false;
            
            if(!$lang)
            	$file	= $yrch->config->get("PATH_CURLOCALE")."/EDITO/".strtolower($ref);
            else
            	$file	= $yrch->config->get("PATH_LOCALE")."/".strtolower($lang)."/EDITO/".strtolower($ref);
            
            if(!file_exists($file))
            {
                if(file_exists($yrch->config->get("PATH_LOCALE")."/".$yrch->i18n_locale($yrch->config->get("DEF_LANG"))."/EDITO/".strtolower($ref)))
                {
                    $file	= $yrch->config->get("PATH_LOCALE")."/".$yrch->i18n_locale($yrch->config->get("DEF_LANG"))."/EDITO/".strtolower($ref);
                }
                else
                {
	            	header("HTTP/1.0 404 Not Found");
	                trigger_error("Document inexistant ($file) !",E_USER_WARNING);
	                return "";
            	}
            }

            $size   = filesize($file);
                       
            if(preg_match("|\.(.*)$|",$ref,$ext))
            {
                $ext    = $ext[1];
                if(in_array($ext,$exts["images"]))
                {
                    $stream = true;
                    @ob_end_clean();
                    header("Content-type: image/".$ext);
                    header("Content-Length: ".$size);

                    include($file);
                    exit();
                }
            }
            
            $fpx    = fopen($file,"r");
            if(!$fpx)
            {
                header("HTTP/1.0 404 Not Found");
                trigger_error(_("Document inexistant !"),E_USER_ERROR);
            }
            
            $source   = "";
            while(!feof($fpx))
            {
                $source   .= fread($fpx,40);
            }
            
            fclose($fpx);
            
            $needle = array(
                "%href=(\'|\\\")([^/]*\.html)\\1%Ui",
                "%src=(\'|\\\")([^/]*\.(".join("|",$exts["images"])."))\\1%Ui"
            );
            
            $replacement    = array(
                "href=\\1edito.php?ref=\\2\\1",
                "src=\\1edito.php?ref=\\2\\1"
            );
            
            return preg_replace($needle,$replacement,$source);
        }
        
    function yrch_select($elts_template, $elts_name, $elts_options, $elts_selected = false, $elts_break = true, $elts_more = "")
    {
    	global  $yrch, $yrch_layout;
	
		if(!is_array($elts_options) || count($elts_options) < 1)
		{
			return false;
		}

		if(!$yrch->config->get("YRCH_TEMPLATESELECT"))
	    {
	    	$yrch_layout->set_file("yrch_select.tpls","structure/yrch_select.tpls");
		    $yrch_layout->set_block("yrch_select.tpls","YRCH_SELECT","yrch_select.blk");
		    $yrch->config->define("YRCH_TEMPLATESELECT",true);
		}

	    $yrch_layout->set_var("yrch_select.blk","");
	    foreach($elts_options as $value => $lib)
	    {	    	
	    	$yrch_layout->set_var(array(
	    			"select_value"	=> $value,
	    			"select_text"	=> $lib,
	    			"select_selected"	=> ($elts_selected == $value) ? "SELECTED" : ""
	    	));
	    	
	    	$yrch_layout->parse("yrch_select.blk","YRCH_SELECT",1);
	    }

		$yrch_layout->set_var(array(
	    		"select_name"	=> $elts_name,
	    		"select_more"	=> $elts_more
	    ));
	    
	    $yrch_layout->parse($elts_template, "yrch_select.tpls");
	    
	    return true;
   	}
  
    function yrch_radio($elts_template, $elts_name, $elts_options, $elts_selected = false, $elts_break = true, $elts_more = "")
    {
    	global  $yrch, $yrch_layout;
	
		if(!is_array($elts_options) || count($elts_options) < 1)
		{
			return false;
		}

		if(!$yrch->config->get("YRCH_TEMPLATERADIO"))
	    {
	    	$yrch_layout->set_file("yrch_radio.tpls","structure/yrch_radio.tpls");
		    $yrch_layout->set_block("yrch_radio.tpls","YRCH_RADIO","yrch_radio.blk");
		    $yrch->config->define("YRCH_TEMPLATERADIO",true);
		}
	    
	    $yrch_layout->set_var("yrch_radio.blk","");
	    foreach($elts_options as $value => $lib)
	    {
	    	if($elts_break)	$lib	= $lib."<br>\n";
	    	
	    	$yrch_layout->set_var(array(
	    			"radio_name"	=> $elts_name,
	    			"radio_value"	=> $value,
	    			"radio_text"	=> $lib,
	    			"radio_more"	=> $elts_more,
	    			"radio_checked"	=> ($elts_selected == $value) ? "CHECKED" : ""
	    	));
	    	
	    	$yrch_layout->parse("yrch_radio.blk","YRCH_RADIO",1);
	    }
	    
	    $yrch_layout->parse($elts_template, "yrch_radio.tpls");
	    
	    return true;
   	}
    
    function yrch_prevnext($elts_template, $elts_index, $elts_count, $elts_perpage, $elts_url)
	{
	    global  $yrch, $yrch_layout;
	
	    $yrch_layout->set_file("yrch_prevnext.tpls","structure/yrch_prevnext.tpls");
	    $yrch_layout->set_block("yrch_prevnext.tpls","PN_ELEMENTS","pn_elements.blk");
	    
	    $nb_pages	= ceil($elts_count / $elts_perpage);
	    $cur_index	= 0;
	    
	    if($nb_pages >= $yrch->config->get("MAX_SEARCH_PAGES"))
        {
        	$start_index	= 	round($elts_index / $elts_perpage) - round($yrch->config->get("MAX_SEARCH_PAGES") / 2);
        	
        	if($start_index < 0) $start_index	= 0;
        }
        else	$start_index	= 0;
        if($nb_pages < 1)	$nb_pages = 1;
		
    	for($i = $start_index; $i < $nb_pages ; $i++)
    	{		
    		if($i >= ($start_index+$yrch->config->get("MAX_SEARCH_PAGES"))) break;
    		
    		$page_index	= urlencode($i*$elts_perpage);
    		$url	    = $elts_url."&index=".$page_index;
    		
    		if($elts_index == $page_index)
    		{
    			$cur	= "font-weight: bold; font-size: 12pt";
    			$cur_index	= $page_index;
    		}
    		else
    		{
    			$cur	= "";	
    		}
    		
    		$yrch_layout->set_var(array(
    			"pn_elements_num"	    => strval($i+1),
    			"pn_elements_url"		=> $url,
    			"pn_elements_cur"	    => $cur
    		));
    		
    		$yrch_layout->parse("pn_elements.blk","PN_ELEMENTS",1);
    	}
	
		$end	= ($nb_pages-1)*$elts_perpage;
		$prev	= $cur_index - $elts_perpage;
		if($prev < 1)	$prev = 1;
		$next	= $cur_index + $elts_perpage;
		if($next > $end)$next = $end;
		
        $yrch_layout->set_var(array(
        		"pn_start"	    => $elts_url."&index=0",
        		"pn_prev"	    => $elts_url."&index=".$prev,
        		"pn_next"	    => $elts_url."&index=".$next,
        		"pn_end"		=> $elts_url."&index=".$end
        ));
        
        $yrch_layout->parse("yrch_prevnext","yrch_prevnext.tpls");
    }
    //end func
    
    function yrch_lang()
    {
	    global $yrch;

	    if(isset($_GET["yrch_lang"]) || isset($_POST["yrch_lang"]))
	    {
	    	//input
	    	if(isset($_GET["yrch_lang"]))		$yrch_lang	= $_GET["yrch_lang"];
	    	elseif(isset($_POST["yrch_lang"]))	$yrch_lang	= $_POST["yrch_lang"];
	    	
	    	$yrch_langs	= array();
			foreach($yrch->list_langs("YRCH") as $lang)
			{
				$yrch_langs[]	= 	$lang["CODE"];
			}
			
			if(in_array($yrch_lang,$yrch_langs))
			{
				return $yrch_lang;	
			}
	    }
	    elseif(isset($_SESSION["yrch_lang"]))
	    {
	    	//session
	    	return $_SESSION["yrch_lang"];
	    }
	   	
	    //defaut
	    if($yrch->config->test("USER_LANG"))
		{
			return $yrch->config->get("USER_LANG");
		}
	    	 
	    if(isset($_SERVER["HTTP_ACCEPT_LANGUAGE"]))
	    {
	    	foreach(explode(',', $_SERVER["HTTP_ACCEPT_LANGUAGE"]) as $accepted)
	    	{
	    		$accepted	= trim($accepted);
	    		foreach($yrch->config->get("YRCH_I18N") as $key => $value)
		    	{
		    		if(eregi('^(' . $value["ACCEPTREG"] . ')(;q=[0-9]\\.[0-9]+)?$', $accepted))
		    		{
		    			return $key;
		    		}
		    	}
		    }
	    }
	    
	    if(isset($_SERVER["HTTP_USER_AGENT"]))
	    {
	    	foreach($yrch->config->get("YRCH_I18N") as $key => $value)
		    {
		    	if(eregi('(\(|\[|;[[:space:]])(' . $value["ACCEPTREG"] . ')(;|\]|\))', $_SERVER["HTTP_USER_AGENT"]))
		    	{
		    		return $key;
		    	}
		    }
	    } 
	    	    
	    return $yrch->config->get("DEF_LANG");
    }
    
    function color_split ( $color )
	{
        $color = str_replace('#', '', $color);
        $c[] = hexdec( substr( $color, 0, 2 ) );
        $c[] = hexdec( substr( $color, 2, 2 ) );
        $c[] = hexdec( substr( $color, 4, 2 ) );
        return $c;
	}
		
	function color_scaled($scale, $color1 = "#FF0000", $color2 = "#50C53A")
	{
		if($scale > 100 || $scale < 0)	trigger_error("l'echelle doit etre comprise entre 0 et 100",E_USER_ERROR);
		
		$color1	= color_split ( $color1 );
		$color2	= color_split ( $color2 );
				
		$red_steps   = ( $color2[0] - $color1[0] ) / 100;
        $green_steps = ( $color2[1] - $color1[1] ) / 100;
        $blue_steps  = ( $color2[2] - $color1[2] ) / 100;
		
		$col[0] = round($red_steps * $scale);
        $col[1] = round($green_steps * $scale);
        $col[2] = round($blue_steps * $scale);
		
		for ( $i = 0; $i < 3; $i++ )
        {
            $partcolor = $color1[$i] + $col[$i];
            // If the color is less than 256
            if (  $partcolor < 256 )
            {
                // Makes sure the colors is not less than 0
                if ( $partcolor > -1 )
                {
                    $newcolor[$i] = $partcolor;
                } else {
                    $newcolor[$i] = 0;
                }
            // Color was greater than 255
            } else {
                $newcolor[$i] = 255;
            }
        }
		
		return	"#".sprintf('%02X%02X%02X',$newcolor[0],$newcolor[1],$newcolor[2]);
	}
?>