<?php
	/*	Yrch! - Hierarchised Website Directories
    Copyright (C) 2002-2003 Guillaume Boddaert <narya@yrch.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.*/
	
	class y_access extends y_mailer
	{
		/**
		* dfinition du degr de sverit des controles
		*
		* La valeur par dfaut est neutre.
		* 1 => neutre
		* 2 => strict
		* 4 => paranoiac
		*
		* @var	int	$policy
		* @see		set_policy
		* @access	private
		*/
		var	$role_policy	= 1;
		
		/**
		* Dfinition des roles autoriss
		*
		* @var	array	$allowed
		* @access	private
		*/
		var	$role_allowed= array();
		
		/**
		* Dfinition des roles interdits
		*
		* @var	array	$rejected
		* @access	private
		*/
		var	$role_denied= array();
		
		var $role_cache	= array();
		
		/**
		* Lister les roles et leurs descriptions
		* 
		* Retourne un tableau de type 	"ID" => "ALIAS"
		*
		* @return	array
		* @access	public
		*/
		function role_list()
		{		
			if($this->is_done("setroles"))	return $this->role_cache;
			
			if(!$this->needs("get_roles",1,"setdb","setconfig"))	return false;
			
			if(!$this->db->squery("ROLE_LIST_ALL"))
				return false;
			
			$roles	= array();
			
			while($this->db->next_record())
			{
				$this->role_cache[$this->db->f("PROFIL_ID")]	= strtoupper($this->db->f("PROFIL_ALIAS"));
			}
			
			$this->done("setroles");
			
			return $this->role_cache;
		}
		
		/**
		* Crer un nouveau role
		*
		* Retourne l'identifiant du role cre
		*
		* @param	string	$alias		Alias du role
		* @param	string	$description	Description du role
		* @return	mixed			l'identifiant du role cre ou false en cas d'erreur
		* @access	public
		*/
		function role_add($alias, $description = "")
		{
			if(!$this->needs("role_add",1,"setdb","setconfig"))	return false;
			
			$id	= $this->db->nextid("SEQ_PROFIL");
			
			if(!$this->db->squery($query,array($id,$alias,$description)))
				return false;
			
			return $id;
		}
		
		/**
		* Effacer un role
		*
		* Cette fonction  supprime le role  ainsi que la  correspondance
		* dans la table des roles/utilisateurs. Cette fonction permet donc
		* de  supprimer   proprement  un   role,  et   de  repercuter  les
		* changements   sur   les   utilisateurs.   Retourne   le   nombre
		* d'utilisateurs impacts.
		*
		* Note :  je recommande  l'utilisation d'un  test strict   la
		* sortie de  cette fonction,  en effet,  0 et  false peuvent  etre
		* retourns et ne signifient pas le mme chose. 0 indique qu'aucun
		* utilisateurs ne faisaient parti du groupe supprim avec  succs,
		* false signifie qu'une erreur est survenue.
		*
		* @param	int	$role		identifiant du role
		* @return	mixed			le nombre d'utilisateurs impact ou faux en cas d'erreur
		* @access	public
		*/
		function role_del($role)
		{
			if(!$this->needs("role_add",1,"setdb","setconfig"))	return false;
			
			if(!$this->db->query("ROLE_DEL",array($role)))
				return false;
			
			$this->db->query($query);
			
			return $this->db->affected_rows();
		}
		
		/**
		* Tous les membres d'un groupe
		* 
		*   Cette  fonction   retourne   un  tableau   d'identifiant
		* d'utilisateur, si $mode est  dfini alors on retourne  un ableau
		* d'user
		*
		* @param	int	$group		identifiant du role
		* @param	boolean	$mode		mode
		* @return	array			tableau d'identifiant ou tableau d'user
		* @access	public
		*/
		function role_members($group, $mode = false)
		{
			if(!$this->needs("role_add",1,"setdb","setconfig"))	return false;
						
			if(!$this->db->squery("ROLE_MEMBERS",array($group)))
				return false;
			
			$return	= array();
			
			while($this->db->next_record())
			{
				if((bool) $mode)
				{										
					$return[$this->db->f("USER_ID")]	= new y_user($this->db->f("USER_ID"));
					$this->fill($return[$this->db->f("USER_ID")]);
				}
				else
					$return[$this->db->f("USER_ID")]	= $this->db->f("USER_ID");
			}
			
			return $return;
		}
		
		/**
		* Dfinir le mode de gestion des droits d'accs 
		*
		* Au choix : "strict" , "neutral" ce qui revient  fonctionner
		* sur  deux  bases  diffrentes,  j'interdis  tout  et j'authorize
		* ensuite, ou j'authorize tous et je bloques certains profils.  Un
		* autre mode "paranoid" est un quivalent de strict mais ne permet
		* pas    l'utilisateur  combinant un  role  autoris  et un  role
		* interdit, l'accs  la page.
		*
		* La valeur par dfaut est NEUTRAL.
		*  
		* Note  : Si  vous appelez  cette fonction  aprs le  premier
		* check()  vous  dclencherez un  nouveau  controle  capable de
		* terminer votre code.
		*
		* Description des modes :
		* "neutral", "n", 1
		* "strict", "s", 2
		* "paranoid", "p", 4
		*
		* @param	$mode	string	description du mode
		* @param	$check	boolean	doit on effectuer un check ?
		* @return	boolean		la dfinition est elle effective ?
		* @access	public
		*/
		function role_policy($mode = 1, $check = true)
		{
			$modes	= array(
					"1"		=> 1,
					"2"		=> 2,
					"4"		=> 4,
					"neutral"	=> 1,
					"strict"	=> 2,
					"paranoid"	=> 4,
					"n"		=> 1,
					"s"		=> 2,
					"p"		=> 4
					);
			
			if(!isset($modes[$mode]))	return false;
			
			//if($this->role_policy == $modes[$mode]) return true;
			
			$this->done("setpolicy");
			$this->role_policy	= $modes[$mode];
			
			if($check && $this->done("rolechecked") && !$this->role_check())
				$this->role_noaccess();

			return true;
		}
		
		/**
		* Remise  zro des regles de droits
		* 
		* Les roles  interdits et  autoriss sont  remis   zros, la
		* regle de gestion (policy) retourne en mode "neutre". 
		* 
		* Suite  l'appel de cette mthode il est ncessaire d'appeler
		* la mthode check() afin de lancer le test.
		*
		* @return	boolean	 l'opration s'est elle passe correctement ?
		*/
		function role_reset()
		{
			$this->role_allowed	= array();
			$this->role_denied	= array();
			$this->role_policy	= 1;
			
			$this->undone("rolechecked");
			return true;
		}
		
		/**
		* Autorise l'accs  un role
		* 
		* Si cette  fonction est  appele avant  le premier check, la
		* fonction  retourne  systmatiquement  vrai,  toutefois  aprs le
		* permier test check() chaque appelle  cette fonction provoque un
		* test. La fonction dclenche automatiquement la page d'erreur  si
		* l'utilisateur ne correspond pas aux regles d'accs dfinies.
		*
		* @param	$role	mixed	un tableau d'alias ou d'identifiant de profils, ou une simple valeur d'alias ou d'identifiant.
		* @param	$check	boolean	doit on effectuer un check ?
		* @return	boolean		Vrai si cela s'est bien pass
		* @access	public
		*/
		function role_allow($role, $check = true)
		{
			if(is_array($role))
			{
				foreach($role as $un_role)
				{
					$this->role_allow($un_role, false);
				}
			}
			else
			{
				if(!in_array($role,$this->role_allowed))
					$this->role_allowed[]	= $role;
			}
			
			if($this->done("rolechecked") && $check)	
				if(!$this->role_check())
					$this->role_noaccess();
			
			return true;
		}
		
		/**
		* Interdit l'accs  un rle
		*
		* @param	$role	mixed	un tableau d'alias ou d'identifiant de profils, ou une simple valeur d'alias ou d'identifiant.
		* @return	boolean		l'opration s'est elle effectue correctement ?
		* @access	public
		*/
		function role_deny($role, $check = true)
		{
			if(is_array($role))
			{
				foreach($role as $un_role)
				{
					$this->role_deny($un_role,false);
				}
			}
			else
			{
				if(!in_array($role,$this->role_denied))
					$this->role_denied[]	= $role;
			}
			
			if($this->done("rolechecked") && $check)	
				if(!$this->role_check())
					$this->no_access();
			
			return true;
		}
			
		/**
		* Lancement d'un controle sur l'utilisateur dfini
		*
		* En   fonction   des   roles   autoriss   (allow()),  et
		* interdits(deny()), en fonction du mode de gestion (set_policy())
		* on  lance  un  test sur  l'utilisateur  pass  en paramtre.  La
		* fonction retourne s'il a accs ou non au reste de du document en
		* fonction des informations amasses.
		* 
		* Note : un  utilisateur doit pralablement tre dfini  sinon
		* la fonction retourne une erreur fatale
		*
		* @return 	boolean
		* @access	private
		*/		
		function role_check()
		{			
			if(!is_subclass_of ($this, "y_user") && get_class($this) != "y_user")
			{
				trigger_error(_("role_check: impossible d'executer cette commande si la classe n'est pas une clase de type utilisateur"),E_USER_WARNING);	
				return false;
			}
			if(!$this->needs("role_check",1,"setid"))	return false;
			
			switch($this->role_policy)
			{
				case "1" : 	
						$check = true;
						foreach($this->role_denied as $deny)
						{
							if($this->is($deny))
								return false;
						}
						break;
				case "2" : 	$check = false;
						foreach($this->role_allowed as $allow)
						{
							if($this->is($allow))
								return true;
						}
						break;
				case "4" : 	$check = false;
						foreach($this->role_allowed as $allow)
						{
							if($this->is($allow))
							{
								$check = true;
								break;
							}
						}
						foreach($this->role_denied as $deny)
						{
							if($this->is($deny))
							{
								$check = false;
								break;
							}
						}
						break;
			}
			
			$this->done("rolechecked");
			
			return $check;
		}
		
		/**
		* Processus de gestion des droits automatique
		* 
		* Cette fonction va chercher les dfinitions d'accs telle que
		* dfinies dans la configuration. Puis fait tourner le  controleur
		* et assume seule l'affichage de l'erreur.
		*
		* Note : elle est utilise par NOYO uniquement.
		*
		* @return	boolean		retourne true ou rien en cas d'interdiction de l'accs (le code est intrrompu)
		* @access	public
		*/
		function role_process()
		{
			$this->role_policy($this->config->get("ACCESS_POLICY"));
			
			$this->role_allow($this->config->get("ACCESS_ALLOW"));
			$this->role_deny ($this->config->get("ACCESS_DENY"));
						
			if($this->role_check())
				return true;
				
			$this->role_noaccess();
		}
		
		/**
		* Fonction sens rendre proprement une interdictiond d'accs
		*
		* @access 	private
		*/
		function role_noaccess()
		{
			if(!$this->needs("role_noaccess",1,"setconfig"))	return false;

			global $yrchuser;

			if(!isset($yrchuser) || $yrchuser->get_id() == 0)
			{
				include($this->config->get("PATH_ROOT")."/include/yrchlogin.inc.php");
			}
			else
			{
				include($this->config->get("PATH_ROOT")."/include/yrchforbid.inc.php");
			}
			exit();
		}
		
		function role_alias2id($what)
		{
			$all_roles	= $this->role_list();
			
			if(!is_numeric($what))
			{
				//cas alias
				if(!in_array(strtoupper($what),$all_roles))
				{
					return false;
				}
				
				$what	= array_search(strtoupper($what),$all_roles);
			}
			else	return $what;
			
			if(!isset($all_roles[$what]))	return false;
			
			return $what;
		}
	}
?>