<?php
	/*	Yrch! - Hierarchised Website Directories
    Copyright (C) 2002-2003 Guillaume Boddaert <narya@yrch.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.*/
    
	class y_user extends y_auth
	{
		var $nickname	= "";
		var $username	= "";
		var $password	= "";
		var $added	= "";
		var $updated 	= "";
		var $status	= "";
		var $email	= "";
		var $user_roles	= array();
		var $user_conf  = array();
		
		function y_user($id = false)
		{
			if($id !== false)	$this->set_id($id);
		}
		
		function get_nickname($force = false)
		{
			if(!$this->is_done("setusernicksession") || $force == true)
			{
				$this->read();
			}
			
			if(!$this->is_done("setnickname"))	
			{				
				return false;
			}
			else	return $this->nickname;
		}
		
		function get_username()
		{
			$this->read();
			
			if(!$this->is_done("setusername"))	
			{				
				return false;
			}
			else	return $this->username;
		}
		
		function get_password()
		{
			$this->read();
						
			if(!$this->is_done("setpassword"))	
			{								
				return false;
			}
			
			return $this->password;
		}
		
		function get_added($format = false)
		{
			if(!$format)	$format	= $this->config->get("DEF_DATEFORMAT");
			
			$this->read();
			
			if(!$this->is_done("setadded"))	
			{				
				return false;
			}
			else	return strftime($format,$this->added);
		}
		
		function get_updated($format = false)
		{
			if(!$format)	$format	= $this->config->get("DEF_DATETIMEFORMAT");
			
			$this->read();
			
			if(!$this->is_done("setupdated"))	
			{				
				return false;
			}
			else	return strftime($format,$this->updated);
		}
		
		function get_status()
		{
			$this->read();
			
			if(!$this->is_done("setstatus"))	
			{				
				return false;
			}
			else	return $this->status;
		}
		
		function get_email()
		{
			$this->read();
			
			if(!$this->is_done("setemail"))	
			{				
				return false;
			}
			else	return $this->email;
		}
		
		function set_nickname($nickname, $force = false)
		{			
			if(!$force)
			{
				if(!eregi("^.{4,30}$",$nickname))
					$this->ierror_add(sprintf(_("\"%s\" est incorrect, votre pseudonyme doit mesurer entre 4 et 30 caractres."),$nickname));
			}
			if(!$this->ierror_ok())	return false;
			
			$this->done("setnickname");
			$this->nickname	= $nickname;
			
			return true;
		}
		
		function set_username($username, $force = false)
		{
			$username	= strtolower($username);
			
			if(!$force)
			{
				$already_exist	= $this->user_exists($username);
				if($already_exist && ($already_exist != $this->id))
					$this->ierror_add(sprintf(_("L'utilisateur '%s' existe dj"),$username));
				if(!eregi("^[_a-z0-9]{5,10}$",$username))
					$this->ierror_add(sprintf(_("l'identifiant '%s' ne peut etre compos que de caractres alphanumriques simples (a->z 0->9 et _) et doit mesurer entre 5 et 10 caractres"),$username));
			}	
			if(!$this->ierror_ok())	return false;
				
			$this->done("setusername");
			$this->username	= strtolower($username);
		}
		
		function set_password($password, $password_revalidate = false, $force = false)
		{			
			if(!$force)
			{
				if(trim($password) == "")
				{
					$this->ierror_add(_("Le mot de passe ne peut tre vide"));
				}
				if($password_revalidate !== false && ($password_revalidate !== $password))
				{
					$this->ierror_add(_("Le deux mots de passe ne sont pas identique"));
				}
				if(!eregi("^.{6,}$",$password))
				{
					$this->ierror_add(_("Le mot de passe doit possder plus de 6 caractres"));
				}
			}
			if(!$this->ierror_ok())	return false;
			
			if($force == false)	$password = md5($password);
			
			$this->password	= $password;
			$this->done("setpassword");
			return true;
		}
		
		function set_status($status, $force = false)
		{
			$status	= strtolower($status);
			if(!$force)
			{
				if(!in_array($status,array("ok", "pending", "locked", "dead")))
					$this->ierror_add(_("Le status est incorrect"));
			}
			
			if(!$this->ierror_ok())	return false;
			
			$this->done("setstatus");
			$this->status	= $status;
		}
		
		function set_email($email, $force = false)
		{
			if(!$force)
			{
				$already_exist	= $this->email_exists($email);
				if($already_exist && ($already_exist != $this->id))
					$this->ierror_add(sprintf(_("L'adresse '%s' est dj rfrence"),$email));
				if(!eregi("^[_a-z0-9-]+(\.[_a-z0-9-]+)*@[a-z0-9-]+(\.[a-z0-9-]+)*$",$email))
					$this->ierror_add(sprintf(_("'%s' est une adresse email invalide"),$email));
			}
			if(!$this->ierror_ok())	return false;
			
			$this->done("setemail");
			$this->email	= $email;
		}
		
		function set_added($added, $force = false)
		{			
			$this->done("setadded");
			$this->added	= $added;
		}
		
		function set_updated($updated, $force = false)
		{
			$this->done("setupdated");
			$this->updated	= $updated;
		}
		
		//Cl d'activation
		function build_akey()
		{
			if(!$this->needs("build_akey",1,"setconfig","setdb","setid")) return false;

			return	md5($this->get_username().$this->get_added("%Y%h").$this->get_nickname());	
		}
		
		/**
		 permet de lire des informations non sensibles en session
		*/
		
		function session_read()
		{
			if(isset($_SESSION["USERDATA"]))
			{
				if(isset($_SESSION["USERDATA"]["NICKNAME"]))
				{
					$this->done("setusernicksession");
					$this->set_nickname($_SESSION["USERDATA"]["NICKNAME"]);
				}
				
				return true;
			}
			$this->session_write();
			return false;
		}
		
		function session_write()
		{
			$_SESSION["USERDATA"]["NICKNAME"]	=	$this->get_nickname();
			return true;
		}
		
		function read()
		{
			if($this->is_done("read"))	return true;
			
			if(!$this->needs("read",1,"setconfig","setdb","setid")) return false;
			//ne pas ecraser si dj dfini
			
			$this->db->squery("USER_READ",array($this->get_id()));
			$this->done("read");
			
			if($this->db->next_record())
			{				
				if(!$this->is_done("setusername"))	
				{
					$this->set_username($this->db->f("USER_NAME"),true);
				}
				if(!$this->is_done("setpassword"))	
				{
					$this->set_password($this->db->f("USER_PASS"),false,true);
				}
				if(!$this->is_done("setnickname"))	
				{
					$this->set_nickname($this->db->f("USER_NICK"),true);
				}
				if(!$this->is_done("setemail"))	
				{
					$this->set_email($this->db->f("USER_EMAIL"),true);
				}
				if(!$this->is_done("setadded"))	
				{
					$this->set_added($this->db->f("USER_ADDED"),true);
				}
				if(!$this->is_done("setupdated"))	
				{
					$this->set_updated($this->db->f("USER_UPDATED"),true);
				}
				if(!$this->is_done("setstatus"))	
				{
					$this->set_status($this->db->f("USER_STATUS"),true);
				}

				return true;	
			}
			return false;
		}
		
		function feed($values)
		{
			if(!is_array($values))
			{
				return false;
			}
			
			$this->done("read");
			
			if(!$this->is_done("setusername"))	$this->set_username($this->db->f("USER_NAME"),true);
			if(!$this->is_done("setpassword"))	$this->set_password($this->db->f("USER_PASS"),false,true);
			if(!$this->is_done("setnickname"))	$this->set_nickname($this->db->f("USER_NICK"),true);
			if(!$this->is_done("setemail"))		$this->set_email($this->db->f("USER_EMAIL"),true);
			if(!$this->is_done("setadded"))		$this->set_added($this->db->f("USER_ADDED"),true);
			if(!$this->is_done("setupdated"))	$this->set_updated($this->db->f("USER_UPDATED"),true);
			if(!$this->is_done("setstatus"))	$this->set_status($this->db->f("USER_STATUS"),true);

			return true;
		}
		
		/**
			dsactive compltement un utilisateur. Different de delete car on CONSERVE une trace de l'utilisateur.
			
			- suppression des roles
			- passage de la gestion du site AU STAFF
			- mise en tat de vrouillage l'utilisateur
		*/
		
		function disable()
		{
			if(!$this->needs("disable",1,"setconfig","setdb","setid")) return false;
			
			foreach($this->get_roles() as $roleid => $role)
			{
				$this->del_role($roleid);
			}
			
			$this->read();
			//CHANGER L'OWNER DE TOUS LES SITES LIES
			foreach($this->get_sites(false) as $siteid)
			{
				$site	= &site_stack($siteid);
				$site->read();
				$site->set_owner(1);
				$site->update();
			}
			
			$this->set_status("dead");
			$this->log->add("USERDISABLE",sprintf("Dsactivation de l'utilisateur \"%s\" (%s)",$this->get_username(),$this->get_id()));
			return $this->update();
		}
		
		/*
			supprime totalement l'utilisateur :
			- suppression de l'utilisateur
			- "			  des ses roles
			- "			  des sites
			- "			  des stats
			- "			  des reviews du site et de l'utilisateur
			- 
		*/
		
		function delete()
		{			
			$this->del_allroles();
			foreach($this->get_sites(true) as $site)
			{
				$site->delete();
			}
			
			$this->db->squery("USER_DELALLREVIEWS",array($this->get_id()));
			$this->db->squery("USER_DEL",array($this->get_id()));
			
			$this->log->add("USERUPDATE",sprintf("Suppression de l'utilisateur \"%s\" (%s)",$this->get_username(),$this->get_id()));
			
			return true;
		}
		
		function update()
		{
			if(!$this->ierror_ok())	return false;
			if(!$this->needs("update",1,"setlog","setconfig","setdb","setemail","setstatus","setpassword","setusername","setnickname")) return false;		
			$this->log->add("USERUPDATE",sprintf("Modification de l'utilisateur \"%s\" (%s)",$this->get_username(),$this->get_id()));
			
			return $this->db->squery("USER_UPDATE",array($this->get_username(),$this->get_password(true),$this->get_nickname(),$this->get_email(),strftime($this->config->get("DB_DATETIMEFORMAT")),$this->get_status(),$this->get_id()));
		}
	
		function create()
		{
			if(!$this->ierror_ok())	return false;
			if(!$this->needs("create",1,"setlog","setconfig","setdb","setemail","setstatus","setpassword","setusername","setnickname")) return false;

			$id	= $this->db->nextid("SEQ_USER");
			
			$this->set_id($id);
						
			$mupo	= $this->db->squery("USER_CREATE",array($this->get_id(),$this->get_username(),$this->get_password(true),$this->get_nickname(),$this->get_email(),strftime($this->config->get("DB_DATETIMEFORMAT")),strftime($this->config->get("DB_DATETIMEFORMAT")),$this->get_status()));
			$this->log->add("USERCREATE",sprintf("Cration de l'utilisateur \"%s\" (%s)",$this->get_username(),$this->get_id()));
			$this->undone("read");
			return	$mupo;
		}
		
		// Activation desactivation
		
		function activate()
		{
			$cur_status	= $this->get_status();
			if($cur_status == 'pending' || $cur_status == 'locked')
			{
				$this->set_status("ok");
				$this->log->add("USERACTIVE",sprintf("Activation de l'utilisateur \"%s\" (%s)",$this->get_username(),$this->get_id()));
				return $this->update();
			}
			return false;
		}
		
		function lock()
		{
			$cur_status	= $this->get_status();
			if($cur_status == 'ok')
			{
				$this->set_status("locked");
				$this->log->add("USERLOCK",sprintf("Vrouillage de l'utilisateur \"%s\" (%s)",$this->get_username(),$this->get_id()));
				return $this->update();
			}
			return false;
		}
		
		// Fonctions lies aux roles
		
		function get_roles()
		{
			if($this->is_done("setuserroles"))
				return $this->user_roles;
				
			if(!$this->needs("get_roles",1,"setconfig","setdb","setid")) return false;
			
			$this->db->squery("USER_PROFILES",array($this->get_id()));
			
			$list	= array();
			while($this->db->next_record())
			{
				$list[$this->db->f("PROFIL_ID")]	= strtoupper($this->db->f("PROFIL_ALIAS"));
			}
			
			krsort($list);
			
			$this->user_roles	= $list;
			$this->done("setuserroles");
			
			return $list;
		}
		
		function get_higher_role()
		{
			$roles	= $this->get_roles();
			return array_shift($roles);	
		}
		
		function get_role_icone()
		{
			return strtolower($this->get_higher_role()).".gif";
		}
		
		function is($what)
		{
			if(!$this->needs("is",1,"setconfig","setdb","setid")) return false;
			
			$roles	= $this->get_roles();
			if(is_numeric($what))
			{
				//cas id
				
				if(isset($roles[$what]))	return true;
				else	return false;
			}
			else
			{
				//cas alias
				if(in_array(strtoupper($what),$roles))	return true;
				else	return false;
			}
		}
		
		function add_role($what)
		{
			if(!$this->needs("add_role",1,"setconfig","setdb","setid")) return false;
			
			$what	= $this->role_alias2id($what);
			
			if($this->is($what))	return true;
			
			
			$this->db->squery("USER_ADDROLE",array($what,$this->get_id()));
			
			$this->undone("setuserroles");
			$this->log->add("USERROLADD",sprintf("Ajout de l'utilisateur \"%s\" (%s) dans le groupe %s",$this->get_username(),$this->get_id(),$what));
			return $this->db->affected_rows();
		}
		
		function del_role($what)
		{
			if(!$this->needs("del_role",1,"setconfig","setdb","setid")) return false;
			
			$what	= $this->role_alias2id($what);
			
			$this->db->squery("USER_DELROLE",array($what,$this->get_id()));
			
			$this->undone("setuserroles");
			$this->log->add("USERROLDEL",sprintf("Suppression de l'utilisateur \"%s\" (%s) du groupe %s",$this->get_username(),$this->get_id(),$what));
			return $this->db->affected_rows();
		}
		
		function del_allroles()
		{
			$this->db->squery("USER_DELALLROLES",array($this->get_id()));
			
			$this->undone("setuserroles");
			$this->log->add("USERROLENO",sprintf("Suppression de l'utilisateur \"%s\" (%s) de tous les groupes",$this->get_username(),$this->get_id()));
			return $this->db->affected_rows();
		}
		
		//
		
		function get_sites($obj = false)
		{
			if(!$this->needs("get_sites",1,"setconfig","setdb","setid")) return false;
			
			$this->db->squery("USER_GETSITES",array($this->get_id()));
						
			$sites	= array();
			$i		= 0;
			while($this->db->next_record())
			{
				if($obj)
				{
					$sites[$i]	= new y_site($this->db->f("SITE_ID"));
					$this->fill($sites[$i]);
					$sites[$i]->feed($this->db->record);
					$i++;
				}
				else	$sites[]	= $this->db->f("SITE_ID");
			}
			
			return $sites;
		}
		
		function get_reviews($obj = false, $site = false, $lang = false, $status = true, $search = false, $index = false, $limit = 10, $orderby = "REVIEW_DATETIME", $order = "DESC")
		{
			if(!$this->needs("get_reviews",1,"setconfig","setdb","setid")) return false;
			
			return $this->list_reviews($obj, $site, $this->get_id(), $lang, $status, $search, $index, $limit, $orderby, $order);
			
			return $reviews;
		}
		
		function userconf_read()
		{
		    if(!$this->needs("userconf_read",1,"setconfig","setdb","setid")) return false;
		    
		    if($this->is_done("readconf"))  return true;
		    
		    if(isset($_SESSION["USERCONF"]))
		    {
		    	$this->user_conf		= $_SESSION["USERCONF"];
		    }
		    else
		    {
			    $this->db->squery("USER_READCONF",array($this->get_id()));
			    
			    while($this->db->next_record())
				{
				    $this->user_conf[strtoupper($this->db->f("USERCONF_KEY"))]   = $this->db->f("USERCONF_VALUE");
				}
				
				$_SESSION["USERCONF"]	= $this->user_conf;
			}

			$this->done("readconf");
			
			return true;
		}
		
		function userconf_get($key)
		{
		    if(!$this->needs("userconf_get",1,"setconfig","setdb","setid")) return false;
		    
		    $this->userconf_read();
		    $key  = strtoupper($key);

		    if(isset($this->user_conf[$key]))
		    {
		        return  $this->user_conf[$key];
		    }
		    return false;
		}
		
		function userconf_set($key, $value)
		{
		    if(!$this->needs("userconf_set",1,"setconfig","setdb","setid")) return false;
		    
		    $key  = strtoupper($key);
		    
		    $this->userconf_read();
		    $this->userconf_unset($key);
		    
		    if($value !== false)
		    {
		        $this->db->squery("USER_WRITECONF",array($this->get_id(), addslashes($key), addslashes(strval($value))));
		        $this->config->define($key,$value);
		    }
		    
		    $this->done("readconf");
		    return true;
		}
		
		function userconf_unset($key)
		{
		    if(!$this->needs("userconf_unset",1,"setconfig","setdb","setid")) return false;
		    
		    $key  = strtoupper($key);
		    
		    return $this->db->squery("USER_DELCONF",array($this->get_id(), addslashes($key)));
		}
		
		function userconf_merge()
		{
		    if(!$this->needs("userconf_merge",1,"setconfig","setdb","setid")) return false;
		    
		    if(!$this->userconf_read()) return false;

		    foreach($this->user_conf as $key => $value)
		    {
		        $this->config->define($key,$value);
		    }
		    return true;
		}
	}
?>